/**
  *  \file game/act-fix.cc
  */

#include "game/action.h"
#include "game/objlist.h"
#include "game/global.h"
#include "game/ship.h"

/*********************** GStarbaseFixRecycleAction ***********************/

/** \class GStarbaseFixRecycleAction
    \brief Starbase shipyard order rule class.

    This class encapsulates all rules for a Fix/Recycle action. */

/** Initialize. */
GStarbaseFixRecycleAction::GStarbaseFixRecycleAction(GPlanet& the_planet)
    : num_own(0), num_foreign(0), the_planet(the_planet)
{
    for (int i = 1; i <= NUM_SHIPS; ++i) {
        GShip& s = getDisplayedTurn().getShip(i);
        if (s.isExistant() && s.getPos() == the_planet.getPos()) {
            if (s.getOwner() == the_planet.getOwner())
                ++num_own;
            else
                ++num_foreign;
        }
    }
}

/** Check whether this starbase can do a "fix" command. */
bool
GStarbaseFixRecycleAction::canFix() const
{
    return num_own + num_foreign != 0;
}

/** Check whether this starbase can do a "recycle" command. */
bool
GStarbaseFixRecycleAction::canRecycle() const
{
    return num_own != 0;
}

/** Enumerate ships for a particular action.
    \param action desired action (sya_Fix, sya_Recycle).
    \param list   output list, should be empty. */
void
GStarbaseFixRecycleAction::enumerateShipsFor(int action, GObjectList& list)
{
    if (action == sya_None)
        return;
    ASSERT(action == sya_Recycle || action == sya_Fix);
    for (int i = 1; i <= NUM_SHIPS; ++i) {
        GShip& s = getDisplayedTurn().getShip(i);
        if (s.isExistant() && s.getPos() == the_planet.getPos())
            if (s.getOwner() == the_planet.getOwner() || action == sya_Fix)
                list.addObject(getDisplayedTurn().getShipType(), s);
    }
}

bool
GStarbaseFixRecycleAction::setAction(int action, int ship_id)
{
    if (action == sya_None) {
        ship_id = 0;
    } else {
        if (ship_id <= 0 || ship_id >= NUM_SHIPS)
            return false;
        if (action != sya_Fix && action != sya_Recycle)
            return false;

        GShip& the_ship = getDisplayedTurn().getShip(ship_id);
        if (!the_ship.isExistant() || the_ship.getPos() != the_planet.getPos())
            return false;
        if (action != sya_Fix && the_ship.getOwner() != the_planet.getOwner())
            return false;
    }
    the_planet.setBaseShipyardOrder(action, ship_id);
    return true;
}
