/**
  *  \file gfx/gfxnull.cc
  */

#include "gfx/gfx.h"

/** \class GfxNullCanvas
    \brief A graphics sink

    A null canvas ignores all requests. This can be used to 'silence'
    output. */

GfxNullCanvas::~GfxNullCanvas()
{ }

void
GfxNullCanvas::drawHLine(int /*x1*/, int /*y1*/, int /*x2*/, uint32 /*color*/, uint8 /*pat*/, alpha_type /*alpha*/)
{ }

void
GfxNullCanvas::drawVLine(int /*x1*/, int /*y1*/, int /*y2*/, uint32 /*color*/, uint8 /*pat*/, alpha_type /*alpha*/)
{ }

void
GfxNullCanvas::drawPixel(int /*x*/, int /*y*/, uint32 /*color*/, alpha_type /*alpha*/)
{ }

void
GfxNullCanvas::drawBar(GfxRect /*rect*/, uint32 /*color*/, uint32 /*bg*/, const GfxFillPattern& /*pat*/, alpha_type /*alpha*/)
{ }

void
GfxNullCanvas::blitSurface(int /*x*/, int /*y*/, GfxSystemSurface /*src*/, GfxRect /*rect*/)
{ }

void
GfxNullCanvas::blitPattern(GfxRect /*rect*/, int /*x*/, int /*y*/, int /*bytes*/_per_line, const uint8* /*data*/, uint32 /*color*/, uint32 /*bg*/, alpha_type /*alpha*/)
{ }

bool
GfxNullCanvas::lock()
{ }

void
GfxNullCanvas::unlock()
{ }

void
GfxNullCanvas::update(GfxRect /*r*/)
{ }

GfxRect
GfxNullCanvas::computeClipRect(GfxRect r)
{
    // tell 'em we clip away everything.
    return GfxRect(r.x, r.y, 0, 0);
}

GfxSystemSurface
GfxNullCanvas::getSystemSurface()
{
    return 0;
}

uint32
GfxNullCanvas::getPixel(int /*x*/, int /*y*/)
{
    return 0;
}
