/**
  *  \file subr.cc
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include <vector>
#include <map>
#include "int/subr.h"
#include "cpluslib/assert.h"

static std::vector<IntSubroutine*>         all_subrs;
static std::map<string_t, IntSubroutine*>  all_named_subrs;

IntSubroutine::IntSubroutine()
    : uid(0), ref_count(0)
{
    // FIXME: currently, we have an intrinsic limit of 2^24 subrs live
    // at the same time (because that's the number of "value" bits in
    // an opcode). We should have a better handling in case this over-
    // flows. It's very unlikely, though: PCC 1.x runs with 2^24 bytes
    // of address space in total, and even in 2.x, 2^24 subrs would
    // amount to 1.2 GB of RAM for just the BCOs, not including their
    // data. We definitely need a way to reclaim freed subrs, though,
    // because we're eating them up quite fast.
    ASSERT(all_subrs.size() < (1L << 24));

    uid = all_subrs.size();
    all_subrs.push_back(this);
}

IntSubroutine::~IntSubroutine()
{
    all_subrs[uid] = 0;
}

IntSubroutine*
IntSubroutine::getSubrByName(string_t name)
{
    std::map<string_t, IntSubroutine*>::iterator i =
        all_named_subrs.find(name);
    if (i == all_named_subrs.end())
        return 0;
    else
        return i->second;
}

IntSubroutine*
IntSubroutine::getSubrByIndex(uint32 id)
{
    if (id >= all_subrs.size())
        return 0;
    return all_subrs[id];
}

void
IntSubroutine::setSubrName(string_t name)
{
    std::map<string_t, IntSubroutine*>::iterator i =
        all_named_subrs.find(name);
    if (i != all_named_subrs.end()) {
        if (i->second == this)
            return;

        i->second->decUseCount();
        // FIXME: delete it?
    }
    all_named_subrs[name] = this;
    incUseCount();
}

