/**
  *  \file ui/skin.h
  */
#ifndef PCC_V2_UI_SKIN_H
#define PCC_V2_UI_SKIN_H

#include "cpluslib/object.h"
#include "gfx/gfx.h"

//
//  Color Management
//

struct UIColor {
    enum Color {
        tc_Static,              ///< static text
        tc_Green,               ///< green (variable)
        tc_Yellow,              ///< yellow (variable)
        tc_Red,                 ///< red (variable)
        tc_White,               ///< white (headings)
        tc_Contrast,            ///< some contrast color (frame)
        tc_Input,               ///< input lines (blue)
        tc_Blue,                ///< blue for shaded list items
        tc_Faded,               ///< gray for faded list items
        tc_Background           ///< background color approximation
    };
    enum { COUNT = tc_Background+1 };
};

/** A set of colors. */
struct UIColorSet {
    uint8 table[UIColor::COUNT];
    // This dummy element is to allow recognition of initialisation
    // errors. You can write 'UIColorSet x = { a,b,c, {} }' and the
    // compiler will tell you when you have too few colors.
    struct dummy { } dummy;

    const uint8 operator[](UIColor::Color i) const { return table[i]; }
};
extern UIColorSet GRAY_COLOR_SET, DARK_COLOR_SET, BLACK_COLOR_SET,
    BRIGHT_GRAY_COLOR_SET;

//
//  Skinning
//

class UISkin : public Object {
    bool  solid;
 protected:
    GfxParameters  parm;
 public:
    typedef UIColor::Color Color;

    UISkin() : solid(true) { }
    virtual ~UISkin();

    void  setSolid(bool solid = true) { this->solid = solid; }
    bool  isSolid() const { return solid; }

    virtual void  startDraw();

    virtual const UIColorSet& getColors() const = 0;
    virtual void        setColor(Color color);
    virtual void        outText(GfxCanvas& can,  int f, int x, int y, string_t text) = 0;
    virtual void        outTextF(GfxCanvas& can, int f, int x, int y, int lim, string_t text) = 0;
    virtual void        drawBackground(GfxCanvas& can, GfxRect r) = 0;

    operator GfxParameters&() { return parm; }

    GfxParameters& getParameters() { return parm; }

    static UISkin& getDefaultSkin();
};

class UIDefaultSkin : public UISkin {
 public:
    UIDefaultSkin();
    ~UIDefaultSkin();

    const UIColorSet& getColors() const;
    void  outText(GfxCanvas& can,  int f, int x, int y, string_t text);
    void  outTextF(GfxCanvas& can, int f, int x, int y, int lim, string_t text);
    void  drawBackground(GfxCanvas& can, GfxRect r);
};

#endif
