/**
  *  \file util/filepars.h
  */
#ifndef PCC_V2_UTIL_FILEPARS_H
#define PCC_V2_UTIL_FILEPARS_H

#include <iosfwd>
#include "cpluslib/string.h"

// Abstract File Parser Class.
class FileParser {
    string_t name;
    char     comment_char;
 public:
    virtual void process(int line_id, string_t line);
    virtual void unparsed(int line_id, string_t line);

    FileParser();
    virtual ~FileParser() { }

    void parseFile(std::istream& in, int line_id = 0);
    bool parseGameFile(string_t filename);

    /** Set file name. As a convenience function, a FileParser can
        store a file name which you can query during process() or
        unparsed(), for example, for error messages. */
    void setFileName(string_t name) { this->name = name; }

    /** Get file name. \returns the file name previously set with
        setFileName(), which see. */
    const string_t& getFileName() const { return name; }

    /** Set comment character. Default is ';'. */
    void setCommentChar(char c) { this->comment_char = c; }

    /** Get comment character. */
    char getCommentChar() const { return comment_char; }

    void trimComments(string_t& line);
};

#endif
